<?php

declare(strict_types=1);

namespace Dedi\SyliusTaxonTypePlugin\Form\Extension;

use Dedi\SyliusTaxonTypePlugin\Form\Type\SelectionAutocompleteChoiceType;
use Dedi\SyliusTaxonTypePlugin\Model\SelectionAwareInterface;
use Dedi\SyliusTaxonTypePlugin\Model\TypedTaxonInterface;
use Doctrine\Common\Collections\ArrayCollection;
use Sylius\Bundle\ProductBundle\Form\Type\ProductType;
use Sylius\Component\Core\Model\ProductInterface;
use Sylius\Component\Core\Model\ProductTaxonInterface;
use Sylius\Component\Core\Repository\ProductTaxonRepositoryInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Symfony\Component\Form\AbstractTypeExtension;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;

class SelectionAwareProductTypeExtension extends AbstractTypeExtension
{
    public function __construct(
        private readonly FactoryInterface $productTaxonFactory,
        private readonly ProductTaxonRepositoryInterface $productTaxonRepository,
    ) {
    }

    /**
     * @inheritdoc
     */
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->addEventListener(FormEvents::POST_SET_DATA, function (FormEvent $event): void {
                $form = $event->getForm();

                $form->add('selections', SelectionAutocompleteChoiceType::class, [
                    'label' => 'dedi_sylius_taxon_type_plugin.ui.selections',
                    'multiple' => true,
                    'required' => false,
                    'getter' => function (SelectionAwareInterface $product) {
                        $selections = new ArrayCollection();
                        foreach ($product->getProductSelectionTaxons() as $selectionTaxon) {
                            $selections->add($selectionTaxon->getTaxon());
                        }

                        return $selections;
                    },
                ]);
            })
            ->addEventListener(FormEvents::SUBMIT, function (FormEvent $event): void {
                /** @var SelectionAwareInterface|ProductInterface $product */
                $product = $event->getData();

                $selections = $product->getSelections();

                if (null !== $product->getId()) {
                    /** @var ProductTaxonInterface[] $productTaxons */
                    $productTaxons = $this->productTaxonRepository->findBy([
                        'product' => $product,
                    ]);
                    foreach ($productTaxons as $productTaxon) {
                        /** @var TypedTaxonInterface $taxon */
                        $taxon = $productTaxon->getTaxon();
                        if ($taxon->getType() === TypedTaxonInterface::SELECTION_TYPE) {
                            if ($selections->contains($taxon)) {
                                $selections->removeElement($taxon);
                            } else {
                                $product->removeProductTaxon($productTaxon);
                            }
                        }
                    }
                }

                foreach ($selections as $selection) {
                    $productTaxon = $this->productTaxonFactory->createNew();
                    $productTaxon->setTaxon($selection);
                    $product->addProductTaxon($productTaxon);
                }
            })
        ;
    }

    /**
     * @inheritdoc
     */
    public static function getExtendedTypes(): iterable
    {
        return [ProductType::class];
    }
}
