<?php

declare(strict_types=1);

namespace Dedi\SyliusTaxonTypePlugin\Form\Type;

use Dedi\SyliusTaxonTypePlugin\Model\TypedTaxonInterface;
use Doctrine\ORM\EntityRepository;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\UX\Autocomplete\Form\AsEntityAutocompleteField;
use Symfony\UX\Autocomplete\Form\BaseEntityAutocompleteType;

#[AsEntityAutocompleteField(
    alias: 'sylius_admin_selection',
    route: 'sylius_admin_entity_autocomplete',
)]
final class SelectionAutocompleteChoiceType extends AbstractType
{
    public function __construct(
        private readonly string $taxonClass,
    ) {
    }

    /**
     * @inheritdoc
     */
    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'class' => $this->taxonClass,
            'choice_name' => 'name',
            'choice_value' => 'code',
            'query_builder' => function (EntityRepository $er) {
                $qb = $er->createQueryBuilder('o');
                $qb->andWhere('o.type = :type');
                $qb->setParameter('type', TypedTaxonInterface::SELECTION_TYPE);

                return $qb;
            },
        ]);
    }

    /**
     * @inheritdoc
     */
    public function buildView(FormView $view, FormInterface $form, array $options): void
    {
        $view->vars['remote_criteria_type'] = 'contains';
        $view->vars['remote_criteria_name'] = 'phrase';
    }

    /**
     * @inheritdoc
     */
    public function getBlockPrefix(): string
    {
        return 'dedi_sylius_taxon_type_selection_autocomplete_choice';
    }

    /**
     * @inheritdoc
     */
    public function getParent(): string
    {
        return BaseEntityAutocompleteType::class;
    }
}
