<?php

declare(strict_types=1);

namespace Dedi\SyliusTaxonTypePlugin\Form\Type;

use Dedi\SyliusTaxonTypePlugin\Form\DataTransformer\ProductTaxonToProductTransformer;
use Sylius\Bundle\ResourceBundle\Form\DataTransformer\RecursiveTransformer;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\UX\Autocomplete\Form\AsEntityAutocompleteField;
use Symfony\UX\Autocomplete\Form\BaseEntityAutocompleteType;

#[AsEntityAutocompleteField(
    alias: 'sylius_admin_taxon_product',
    route: 'sylius_admin_entity_autocomplete',
)]
final class TaxonProductAutocompleteChoiceType extends AbstractType
{
    public function __construct(
        private readonly FactoryInterface $productTaxonFactory,
        private readonly RepositoryInterface $productTaxonRepository,
        private readonly RepositoryInterface $taxonRepository,
        private readonly string $productClass,
    ) {
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'class' => $this->productClass,
            'choice_name' => 'name',
            'choice_value' => 'code',
        ]);
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        if (true === $options['multiple']) {
            $builder->addModelTransformer(
                new RecursiveTransformer(
                    new ProductTaxonToProductTransformer(
                        $this->productTaxonFactory,
                        $this->productTaxonRepository,
                        $this->taxonRepository,
                        $options['extra_options']['taxon_id'],
                    ),
                ),
            );
        } else {
            $builder->addModelTransformer(
                new ProductTaxonToProductTransformer(
                    $this->productTaxonFactory,
                    $this->productTaxonRepository,
                    $this->taxonRepository,
                    $options['extra_options']['taxon_id'],
                ),
            );
        }
    }

    public function getBlockPrefix(): string
    {
        return 'dedi_sylius_taxon_type_taxon_product_autocomplete_choice';
    }

    public function getParent(): string
    {
        return BaseEntityAutocompleteType::class;
    }
}
