<?php

declare(strict_types=1);

namespace Dedi\SyliusTaxonTypePlugin\Twig\Component\Taxon;

use Dedi\SyliusTaxonTypePlugin\Model\TypedTaxonInterface;
use Sylius\Bundle\AdminBundle\Generator\TaxonSlugGeneratorInterface;
use Sylius\Bundle\UiBundle\Twig\Component\ResourceFormComponentTrait;
use Sylius\Bundle\UiBundle\Twig\Component\TemplatePropTrait;
use Sylius\Component\Core\Model\TaxonInterface;
use Sylius\Resource\Doctrine\Persistence\RepositoryInterface;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\UX\Autocomplete\Checksum\ChecksumCalculator;
use Symfony\UX\LiveComponent\Attribute\AsLiveComponent;
use Symfony\UX\LiveComponent\Attribute\LiveAction;
use Symfony\UX\LiveComponent\Attribute\LiveArg;
use Symfony\UX\LiveComponent\Attribute\LiveProp;
use Symfony\UX\LiveComponent\LiveCollectionTrait;
use Symfony\UX\TwigComponent\Attribute\ExposeInTemplate;

#[AsLiveComponent]
class FormComponent
{
    use LiveCollectionTrait;

    /** @use ResourceFormComponentTrait<TaxonInterface> */
    use ResourceFormComponentTrait;

    use TemplatePropTrait;

    #[LiveProp(hydrateWith: 'hydrateType', dehydrateWith: 'dehydrateType', fieldName: 'type')]
    public string $type = TypedTaxonInterface::DEFAULT_TYPE;

    public function __construct(
        RepositoryInterface $repository,
        FormFactoryInterface $formFactory,
        string $resourceClass,
        string $formClass,
        private readonly TaxonSlugGeneratorInterface $slugGenerator,
        private readonly ChecksumCalculator $checksumCalculator,
    ) {
        $this->initialize($repository, $formFactory, $resourceClass, $formClass);
    }

    protected function instantiateForm(): FormInterface
    {
        /** @var TypedTaxonInterface $resource */
        $resource = $this->resource;
        $resource->setType($this->type);

        return $this->formFactory->create($this->formClass, $resource);
    }

    public function hydrateType(?string $value): string
    {
        if (null === $value) {
            return TypedTaxonInterface::DEFAULT_TYPE;
        }

        return $value;
    }

    public function dehydrateType(?string $type): ?string
    {
        return $type;
    }

    #[LiveAction]
    public function generateTaxonSlug(#[LiveArg] string $localeCode): void
    {
        $name = $this->formValues['translations'][$localeCode]['name'];
        $parent = $this->repository->findOneBy(['code' => $this->formValues['parent']]);

        $this->formValues['translations'][$localeCode]['slug'] = $this->slugGenerator->generate($name, $localeCode, $parent);
    }

    #[ExposeInTemplate(name: 'extra_options')]
    public function getExtraOptions(): string
    {
        return base64_encode(json_encode(
            [
                'taxon_id' => $this->resource->getId(),
                '@checksum' => $this->checksumCalculator->calculateForArray(['taxon_id' => $this->resource->getId()]),
            ],
        ));
    }
}
